<?php

require_once ("Files.interface.inc");
require_once ("vmlib/phpUtil.inc");
require_once ("vmlib/logUtil.inc");
require_once ("aws_sdk/aws-autoloader.php");

use Aws\Credentials\CredentialProvider;

class S3_files implements Files{

    private $oProperties;
    private $oS3Client;

    function __construct($properties){
        $this->oProperties = $properties;

        $oProvider = CredentialProvider::ini($properties['fileS3UploaderProfil'], $properties['AWSCredentialsFilePath']);
        $oProvider = CredentialProvider::memoize($oProvider);
        // create client s3
        $this->oS3Client = new Aws\S3\S3Client(array(
            'version'=>'latest',
            'region'=> $properties['fileS3UploaderRegion'],
            //'profile'=> $properties['fileS3UploaderProfil'],
            'credentials' => $oProvider,
            'debug' => false
        ));
    }
    /**
     *parse vitis properties ti extract bucket name and bucket prefix.
     *@private
     *@return array bucket'name, prefix.
     */
    private function getBucketConst(){
        $sBucket = $this->oProperties['fileS3UploaderBucket'];
        $sPrefix = "";

        if (strpos($sBucket, "/") > -1){
            $aBucket = explode("/", $sBucket );
            $sBucket = $aBucket[0];
            $sPrefix = implode("/", array_slice($aBucket, 1));
        }

        return array($sBucket, $sPrefix);
    }
    /**
     *This method return the extension of a file.
     *@param string $sFilePath File's path.
     *@return string|boolean hash md5 of file content or false if the file doesn't exist.
     */
    public function getFileEtag($sFilePath){
        list($sBucket, $sPrefix) = $this->getBucketConst();
        $sFilePath = str_replace($this->oProperties["vas_home"], $sPrefix , $sFilePath);

        try {
            $oResult = $this->oS3Client->headObject(array(
                'Bucket' => $sBucket,
                'Key'    => $sFilePath)
            );

            return str_replace('"', '', $oResult["ETag"]);
        }catch (Aws\S3\Exception\S3Exception $e){
            writeToErrorLog($e->getMessage());
            return false;
        }
    }
    /**
     *verify if a file exists.
     *@param string $sFilePath File's path.
     *@return boolean true if file exists, false else.
     */
    public function file_exists ($sFilePath){

        list($sBucket, $sPrefix) = $this->getBucketConst();
        $sFilePath = str_replace($this->oProperties["vas_home"], $sPrefix , $sFilePath);

        try {
            $this->oS3Client->headObject(array(
                'Bucket' => $sBucket,
                'Key'    => $sFilePath)
            );
        }catch (Aws\S3\Exception\S3Exception $e){
            writeToErrorLog($e->getMessage());
            return false;
        }
        return true;
    }
    /**
     *Write a string in a file.
     *@param string $sFilePath File's path.
     *@param string $sData string to write in this file.
     *@param integer $iFlags flags to write in file (http://php.net/manual/fr/function.file-put-contents.php) Default : 0.
     *@param resource $mContext result de stream_context_create().
     *@return integer|boolean number of byte written, or false if the process encounter an error.
     */
    public function file_put_contents ($sFilePath, $sData, $iFlags = 0, $mContext = null){
        list($sBucket, $sPrefix) = $this->getBucketConst();
        $sFilePath = str_replace($this->oProperties["vas_home"], $sPrefix , $sFilePath);

        try {
            $this->oS3Client->putObject(array(
                'Bucket' => $sBucket,
                'Key'    => $sFilePath,
                'Body'   => $sData,
            )
        );
        }catch (Aws\S3\Exception\S3Exception $e){
            writeToErrorLog($e->getMessage());
            return false;
        }
        return strlen($sData);
    }
    /**
     *read a file and return content in a string.
     *@param string $sFilePath File's path.
     *@param boolean $bUseIncludePath use include var to resolve a relative path.
     *@param resource $mContext result de stream_context_create().
     *@param integer $iOffset offset in bytes to begin file reading, Default : 0.
     *@param integer $iMaxLen Number of byte to read from the file, Default : -1 = read until the file's end.
     *@return string|boolean file content, or false if the process encounter an error.
     */
    public function file_get_contents ($sFilePath, $bUseIncludePath = FALSE, $mContext = null, $iOffset = 0, $iMaxLen = -1){

        list($sBucket, $sPrefix) = $this->getBucketConst();
        $sFilePath = str_replace($this->oProperties["vas_home"], $sPrefix , $sFilePath);

        $sPath = $this->oProperties['extract_dir'] . "/" . getUniqRandomId();

        try{
            $this->oS3Client->getObject(array(
              'Bucket' => $sBucket,
              'Key' => $sFilePath,
              'SaveAs' => $sPath
            ));
        }catch (Aws\S3\Exception\S3Exception $e){
            writeToErrorLog($e->getMessage());
            unlink($sPath);
            return false;
        }

        $sFile = "";

        if ($iMaxLen == -1){
            $sFile =  file_get_contents ($sPath, $bUseIncludePath, null, $iOffset);
        } else {
            $sFile =  file_get_contents ($sPath, $bUseIncludePath, null, $iOffset, $iMaxLen);
        }
        unlink($sPath);
        return $sFile;
    }
    /**
     *read a file and return content in a string if eTag matches.
     *@param string $sFilePath File's path.
     *@param string $sEtag Hash Sha1 of file's content.
     *@param boolean $bUseIncludePath use include var to resolve a relative path.
     *@param resource $mContext result de stream_context_create().
     *@param integer $iOffset offset in bytes to begin file reading, Default : 0.
     *@param integer $iMaxLen Number of byte to read from the file, Default : -1 = read until the file's end.
     *@return string|boolean file content, or false if the process encounter an error.
     */
    public function file_get_contents_with_etag ($sFilePath, $sEtag ,$bUseIncludePath = FALSE, $mContext = null, $iOffset = 0, $iMaxLen = -1){
        list($sBucket, $sPrefix) = $this->getBucketConst();
        $sFilePath = str_replace($this->oProperties["vas_home"], $sPrefix , $sFilePath);

        $sPath = $this->oProperties['extract_dir'] . "/" . getUniqRandomId();

        try{
            $this->oS3Client->getObject(array(
              'Bucket' => $sBucket,
              'Key' => $sFilePath,
              'IfMatch' => $sEtag,
              'SaveAs' => $sPath
            ));
        }catch (Aws\S3\Exception\S3Exception $e){
            writeToErrorLog($e->getMessage());
            unlink($sPath);
            return false;
        }
        $sFile = "";

        if ($iMaxLen == -1){
            $sFile =  file_get_contents ($sPath, $bUseIncludePath, null, $iOffset);
        } else {
            $sFile =  file_get_contents ($sPath, $bUseIncludePath, null, $iOffset, $iMaxLen);
        }
        unlink($sPath);
        return $sFile;
    }
    /**
     *copy a file in the same bucket.
     *@param string $sSourceFilePath Object's key to copy.
     *@param string $sDestFilePath Object's destination key.
     *@return boolean true on sucess or false if the process encounter an error.
     */
    public function copy ($sSourceFilePath, $sDestFilePath){
        list($sBucket, $sPrefix) = $this->getBucketConst();
        $sDestFilePath = str_replace($this->oProperties["vas_home"], $sPrefix , $sDestFilePath);
        $sSourceFilePath = str_replace($this->oProperties["vas_home"], $sPrefix , $sSourceFilePath);

        try{
            $this->oS3Client->copyObject(array(
              'Bucket' => $sBucket,
              'Key' => $sDestFilePath,
              'CopySource' => $sBucket . "/" . $sSourceFilePath
            ));
            return true;
        }catch (Aws\S3\Exception\S3Exception $e){
            writeToErrorLog($e->getMessage());
            return false;
        }
    }
    /**
     *copy a file in another bucket.
     *@param string $sSourceFilePath Object's key to copy.
     *@param string $sDests3Key Object's destination key.
     *@param string $sDestBucket Object's destination bucket.
     *@return boolean true on sucess or false if the process encounter an error.
     */
    public function copyInAnotherBucket ($sSourceFilePath, $sDests3Key, $sDestBucket){
        list($sBucket, $sPrefix) = $this->getBucketConst();
        $sSourceFilePath = str_replace($this->oProperties["vas_home"], $sPrefix , $sSourceFilePath);

        try{
            $this->oS3Client->copyObject(array(
              'Bucket' => $sDestBucket,
              'Key' => $sDests3Key,
              'CopySource' => $sBucket . "/" . $sSourceFilePath
            ));
            return true;
        }catch (Aws\S3\Exception\S3Exception $e){
            writeToErrorLog($e->getMessage());
            return false;
        }
    }
    /**
     *send a local file to S3.
     *@param string $sSourceFilePath File's path to copy.
     *@param string $sDestFilePath Object's destination key.
     *@return boolean true on sucess or false if the process encounter an error.
     */
    public function copyLocalToS3 ($sSourceFilePath, $sDestFilePath){
        list($sBucket, $sPrefix) = $this->getBucketConst();
        $sDestFilePath = str_replace($this->oProperties["vas_home"], $sPrefix , $sDestFilePath);

        try {
            $this->oS3Client->putObject(array(
                'Bucket' => $sBucket,
                'Key'    => $sDestFilePath,
                'Body'   => file_get_contents($sSourceFilePath),
            )
        );
        }catch (Aws\S3\Exception\S3Exception $e){
            writeToErrorLog($e->getMessage());
            return false;
        }
        return true;
    }
    /**
     *return the size of a file in bytes.
     *@param string $sFilePath File's path.
     *@return integer|boolean File's size in bytes or false if the process encounter an error.
     */
    public function filesize ($sFilePath){
        list($sBucket, $sPrefix) = $this->getBucketConst();
        $sFilePath = str_replace($this->oProperties["vas_home"], $sPrefix , $sFilePath);

        try {
            $oResult = $this->oS3Client->headObject(array(
                'Bucket' => $sBucket,
                'Key'    => $sFilePath)
            );

            return $oResult["ContentLength"];
        }catch (Aws\S3\Exception\S3Exception $e){
            writeToErrorLog($e->getMessage());
            return false;
        }
    }
    /**
     *send file's content into stdout.
     *@param string $sFilePath File's path.
     *@param boolean $bUseIncludePath use include var to resolve a relative path.
     *@param resource $mContext result de stream_context_create().
     *@return integer|boolean File's size in bytes or false if the process encounter an error.
     */
    public function readfile ($sFilePath, $bUseIncludePath = FALSE, $mContext = null){
        list($sBucket, $sPrefix) = $this->getBucketConst();
        $sFilePath = str_replace($this->oProperties["vas_home"], $sPrefix , $sFilePath);

        $sPath = $this->oProperties['extract_dir'] . "/" . getUniqRandomId();

        try{
            $this->oS3Client->getObject(array(
              'Bucket' => $sBucket,
              'Key' => $sFilePath,
              'SaveAs' => $sPath
            ));
        }catch (Aws\S3\Exception\S3Exception $e){
            writeToErrorLog($e->getMessage());
            unlink($sPath);
            return false;
        }

        $iLength = readfile($sPath, $bUseIncludePath, $mContext);
        unlink($sPath);
        return $iLength;
    }
    /**
     *move a file.
     *@param string $sSourceFilePath File's path to copy.
     *@param string $sDestFilePath File's destination.
     *@param resource $mContext result de stream_context_create().
     *@return boolean true on sucess or false if the process encounter an error.
     */
    public function rename ($sSourceFilePath, $sDestFilePath, $mContext = null){
        $bReturn = $this->copy($sSourceFilePath, $sDestFilePath);
        if ($bReturn)
            $bReturn = $this->unlink($sSourceFilePath, $mContext);
        return $bReturn;
    }
    /**
     *remove a file.
     *@param string $sFilePath File's path.
     *@param resource $mContext result de stream_context_create().
     *@return boolean true on sucess or false if the process encounter an error.
     */
    public function unlink ($sFilePath, $mContext = null){
        list($sBucket, $sPrefix) = $this->getBucketConst();
        $sFilePath = str_replace($this->oProperties["vas_home"], $sPrefix , $sFilePath);

        try{
            $this->oS3Client->deleteObject(array(
              'Bucket' => $sBucket,
              'Key' => $sFilePath
            ));
            return true;
        }catch (Aws\S3\Exception\S3Exception $e){
            writeToErrorLog($e->getMessage());
            return false;
        }
    }
    /**
     *return the number of second UNIX of the last modification on a file.
     *@param string $sFilePath File's path.
     *@return integer|boolean timestamp on sucess or false if the process encounter an error.
     */
    public function filemtime ($sFilePath){
        list($sBucket, $sPrefix) = $this->getBucketConst();
        $sFilePath = str_replace($this->oProperties["vas_home"], $sPrefix , $sFilePath);

        try {
            $oResult = $this->oS3Client->headObject(array(
                'Bucket' => $sBucket,
                'Key'    => $sFilePath)
            );

            return $oResult["LastModified"]->format("U");
        }catch (Aws\S3\Exception\S3Exception $e){
            writeToErrorLog($e->getMessage());
            return false;
        }
    }
    /**
     *return the fomrated date of the last modification on a file.
     *@param string $sFilePath File's path.
     *@param string $sFormat Format to convert the timestamp.
     *@return string|boolean date string on sucess or false if the process encounter an error.
     */
    public function filemtime_formated ($sFilePath, $sFormat){
        return date ($sFormat, $this->filemtime($sFilePath));
    }
    /**
     *return the metadata structure of a file.
     *@param string $sFilePath File's path.
     *@return array filename, size, lastModification, href.
     */
    public function getFileInfos ($sFilePath){
        $iFileSize = $this->filesize($sFilePath);

        $sFileSize = $iFileSize . " octets";
        if($iFileSize > 1024){
            if($iFileSize > 1024*1024){
                if($iFileSize > 1024*1024*1024){
                    $sFileSize = "-";
                } else {
                    $sFileSize = (ceil($iFileSize/(1024*1024))) . "Mo";
                }
            }else{
                $sFileSize = (ceil($iFileSize/1024)) . "Ko";
            }
        }

        $aFileName = explode("/", $sFilePath);

        return array("filename"=>$aFileName[count($aFileName) - 1], "size"=>$sFileSize, "lastModification"=> $this->filemtime_formated ($sFilePath, "d/m/Y H:i:s"), "href" => $this->getProxyPassUrl($sFilePath));
    }
    /**
     *return an url to download the file through Vitis API .
     *@param string $sFilePath File's path.
     *@return string url.
     */
    public function getProxyPassUrl ($sFilePath){
        $sEtag = $this->getFileEtag($sFilePath);
        list($sBucket, $sPrefix) = $this->getBucketConst();
        $sFilePath = str_replace($this->oProperties["vas_home"], $sPrefix , $sFilePath);
        $date = new DateTime();
        $sDataUrl = $this->oProperties['web_server_name'] . "/rest/vitis/file_downloader?key=[KEY]&eTag=[ETAG]&d=" . $date->getTimestamp();

        $sFileUrl = str_replace("[KEY]", str_replace($this->oProperties['ws_data_dir'], "ws_data" , $sFilePath), $sDataUrl);
        $sFileUrl = str_replace("[ETAG]", $sEtag, $sFileUrl);

        return $sFileUrl;
    }
    /**
     *scan a directory with a depth of 1.
     *@param string $sDirPath Directory's path.
     *@param integer $iSortOrder flag (http://php.net/manual/fr/function.scandir.php).
     *@param resource $mContext result de stream_context_create().
     *@return array list of file and subdirectoy .
     */
    public function scandir($sDirPath, $iSortOrder = SCANDIR_SORT_ASCENDING, $mContext = null){
        $aReturn = array(".", "..");
        // traitement du bucket et de ses sous-dossiers
       list($sBucket, $sPrefix) = $this->getBucketConst();
       $sListPrefix = str_replace($this->oProperties["vas_home"], $sPrefix , $sDirPath);
      // Suppression du slash de début de ligne (sinon création d'un répertoire vide sur S3).
      if (strpos($sListPrefix, '/') === 0){
          $sListPrefix = substr($sListPrefix, 1);
      }
      if (substr($sListPrefix, -1) !== "/"){
          $sListPrefix .= "/";
      }
       try{
           $aList = $this->oS3Client->listObjectsV2(array(
               'Bucket' => $sBucket,
               'Prefix' => $sListPrefix
           ));

            if ($aList["KeyCount"] > 0){
               foreach ($aList["Contents"] as $aObject){
                   $sObjectPath = str_replace($sListPrefix, "", $aObject["Key"]);
                   $aObjectPath = explode("/", $sObjectPath);
                   if(!empty($aObjectPath[0])){
                       array_push($aReturn, $aObjectPath[0]);
                   }
               }
            }
            if($iSortOrder === SCANDIR_SORT_ASCENDING){
                sort($aReturn);
            }
            if($iSortOrder === SCANDIR_SORT_DESCENDING){
                rsort($aReturn);
            }
            return $aReturn;
        }catch(Aws\S3\Exception\S3Exception $e){
            writeToErrorLog($e->getMessage());
            return false;
        }
    }
    public function mkdir($sDirPath, $iMode = 0777, $bRecursive = FALSE, $mContext = null){
        // ne sert à rien dans un stockage d'objet
        return false;
    }
    public function rmdir($sDirPath, $mContext = null){
        // ne sert à rien dans un stockage d'objet (pas de dossier vide ou peu favoriser clearDir)
        return false;
    }
    /**
     *define if the path is a directory or a file .
     *@param string $sDirPath directory's path.
     *@return boolean true if it's a dir, false else.
     */
    public function is_dir($sDirPath){
        // traitement du bucket et de ses sous-dossiers
       list($sBucket, $sPrefix) = $this->getBucketConst();
       $sListPrefix = str_replace($this->oProperties["vas_home"], $sPrefix , $sDirPath);
      // Suppression du slash de début de ligne (sinon création d'un répertoire vide sur S3).
      if (strpos($sListPrefix, '/') === 0){
          $sListPrefix = substr($sListPrefix, 1);
      }
       try{
           $aList = $this->oS3Client->listObjectsV2(array(
               'Bucket' => $sBucket,
               'Prefix' => $sListPrefix
           ));

           if ($aList["KeyCount"] > 1){
               return true;
           } else if ($aList["KeyCount"] == 1){
               $iPrefixPathCount = count(explode("/", $sListPrefix));
               $iKeyCount = count(explode("/", $aList["Contents"][0]["Key"]));
               if ($iKeyCount > $iPrefixPathCount){
                   return true;
               } else {
                   return false;
               }

           } else {
               return false;
           }
        }catch(Aws\S3\Exception\S3Exception $e){
            writeToErrorLog($e->getMessage());
            return false;
        }
    }
    /**
     *scan a directory without depth limitation and return metadata usefull for treeview.
     *@param string $sDirPath Directory's path.
     *@return array list of file and subdirectory .
     */
    public function getFolderInfos($sDirPath){
         if($this->is_dir($sDirPath)){
             $aContentDir = $this->scandir($sDirPath);
             $aTree = array(
                     "filename"=> end(explode("/",$sDirPath)),
                     "lastModification" => "none",
                     "size"=> "",
                     "content" => array()
             );
             for ($i =0 ; $i < count($aContentDir); $i++){
                 if($aContentDir[$i] !== "." && $aContentDir[$i] !== ".."){
                     array_push($aTree["content"], $this->getFolderInfos($sDirPath . "/" . $aContentDir[$i]));
                 }
             }

             return $aTree;
         } else {
             return $this->getFileInfos($sDirPath);
         }
    }
}


?>
